//
//  FGHudDisplayCell.m
//  FGBseHUD
//
//  Created by Francesco on 2/18/06.
//
//  Copyright 2006 Softech di Germinara Francesco
//
//  www.germinara.it
//  info@germinara.it
//
//  All rights reserved.
/////////////


#import "FGHudDisplayCell.h"
#import "FGNSBezierPathAddition.h"

@implementation FGHudDisplayCell

- (id)init{
    self = [super init];
    if (self) {
	    aImageLeft=[[NSImage  imageNamed:@"FGDisplayLeft.tiff"] retain];
        aImageMiddle=[[NSImage  imageNamed:@"FGDisplayMiddle.tiff"] retain];
		aImageRight=[[NSImage  imageNamed:@"FGDisplayRight.tiff"] retain];	
		theTitleRow1=[[NSString alloc] initWithString:@""];
		theTitleRow2=[[NSString alloc] initWithString:@""];		
		bHaveNumericDisplay=NO;
		nValore=0;
    }
    return self;
}

-(void) dealloc {
	[aImageLeft release];
	[aImageMiddle release];
	[aImageRight release];
	[theTitleRow1 release];
	[theTitleRow2 release];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"FGHudLevelIndicatorValueChanged" object:nil];
	[super dealloc];
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
 NSRect rcInvalidate= [[self controlView] bounds];
 aHudView=(FGHUDView *)[[super controlView] superview];
 
  NSRect rcLeft,rcMiddle,rcRight;
  int nAmpImg=20;
  int nAltImg=47;

  foreColor=[aHudView clrTitleBar];
  backColor=[aHudView clrBodyBar];

  [backColor set];
  NSRectFill(rcInvalidate);
  
   NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:rcInvalidate cornerRadius:4];
   [path setFlatness:1.0];
   [path setLineWidth:1.0];
   [path fill];
 
  
  cellFrame=NSInsetRect(cellFrame,2,2);
  rcLeft=cellFrame;
  rcLeft.size.width=nAmpImg;
  
  rcMiddle=cellFrame;
  rcMiddle.origin.x= rcLeft.size.width;
  rcMiddle.size.width=cellFrame.size.width - (2*nAmpImg);
  
  rcRight=cellFrame;
  rcRight.origin.x= rcMiddle.size.width+rcLeft.size.width;
  rcRight.size.width=nAmpImg;
  
  [aImageLeft setFlipped: YES];
  [aImageLeft drawInRect:rcLeft fromRect:NSMakeRect(0,0,nAmpImg,nAltImg) operation:NSCompositeSourceOver fraction:1.0];		  
  [aImageLeft setFlipped: NO];

  [aImageMiddle setFlipped: YES];
  [aImageMiddle drawInRect:rcMiddle fromRect:NSMakeRect(0,0,1,nAltImg) operation:NSCompositeSourceOver fraction:1.0];		  
  [aImageMiddle setFlipped: NO];

 
  [aImageRight setFlipped: YES];
  [aImageRight drawInRect:rcRight fromRect:NSMakeRect(0,0,nAmpImg,nAltImg) operation:NSCompositeSourceOver fraction:1.0];		  
  [aImageRight setFlipped: NO];
  
  if([theTitleRow1 length]>0){
   [self drawTextRow1:cellFrame];
  }

if([theTitleRow2 length]>0){
  [self drawTextRow2:cellFrame];
 }

if(bHaveNumericDisplay)
 [self drawNumericDisplay:nValore];

}

-(void) drawTextRow1:(NSRect)cellFrame{
 NSString *infoString = [NSString stringWithFormat:@"<span style=\"font-size: 14px; font-family: 'Lucida Grande'; color: #00006d;\"><strong>%@</strong></span>",theTitleRow1];
 NSAttributedString * attributedInfoString = [[NSAttributedString alloc] initWithHTML:[infoString dataUsingEncoding:NSASCIIStringEncoding] documentAttributes:NULL];
 NSPoint stringPoint;
 stringPoint.x =  cellFrame.origin.x=8;
 
 if([theTitleRow2 length]==0)
  stringPoint.y =  cellFrame.origin.y+ (cellFrame.size.height-[attributedInfoString size].height)/2;
 else
  stringPoint.y =  cellFrame.origin.y+cellFrame.size.height-[attributedInfoString size].height-4;
 [attributedInfoString drawAtPoint:stringPoint];
 [attributedInfoString release];
}

-(void) drawTextRow2:(NSRect)cellFrame{
 NSString *infoString = [NSString stringWithFormat:@"<span style=\"font-size: 10px; font-family: 'Lucida Grande'; color: #00be00;\"><strong>%@</strong></span>",theTitleRow2];
 NSAttributedString * attributedInfoString = [[NSAttributedString alloc] initWithHTML:[infoString dataUsingEncoding:NSASCIIStringEncoding] documentAttributes:NULL];
 NSPoint stringPoint;
 stringPoint.x =  cellFrame.origin.x+8;
 if([theTitleRow1 length]==0)
  stringPoint.y =  cellFrame.origin.y+ (cellFrame.size.height-[attributedInfoString size].height)/2;
 else
  stringPoint.y =  cellFrame.origin.y+4;
 [attributedInfoString drawAtPoint:stringPoint];
 [attributedInfoString release];
}

- (void)drawNumericDisplay:(int)valore
{
    NSRect rcBorder;
    NSRect rcBounds= [[self controlView] bounds];
	NSPoint pt;
	pt.x=rcBounds.size.width-50-6;
	pt.y=rcBounds.origin.y+(rcBounds.size.height-22)/2;
		
	rcBorder.origin.x=pt.x-2;
	rcBorder.origin.y=pt.y-2;
	rcBorder.size.width=(20*2)+4;
	rcBorder.size.height=22+4;
	
	[backColor set];
    NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:rcBorder cornerRadius:4];
    [path setFlatness:1.0];
    [path setLineWidth:1.0];
    [path fill];
	
	[self drawNumericDisplayDecimalDigit:valore/10];
	[self drawNumericDisplayUnitDigit:valore % 10];
}
- (void)drawNumericDisplayDecimalDigit:(int)num
{
    NSRect rcBounds= [[self controlView] bounds];
	NSPoint pt;
	pt.x=rcBounds.size.width-50-6;
	pt.y=rcBounds.origin.y+(rcBounds.size.height-22)/2;
	
	NSImage	*numImage;
	if(nValore>=10)
	 numImage = [NSImage imageNamed:[NSString stringWithFormat:@"numero-%d", num]];
	else 
	 numImage = [NSImage imageNamed:[NSString stringWithFormat:@"numero-bkg.png", num]];
	[numImage compositeToPoint:pt operation: NSCompositeSourceOver];
}
- (void)drawNumericDisplayUnitDigit:(int)num
{
    NSRect rcBounds= [[self controlView] bounds];
	NSPoint pt;
	pt.x=rcBounds.size.width-30-6;
	pt.y=rcBounds.origin.y+(rcBounds.size.height-22)/2;
	NSImage	*numImage;
	if(nValore>0)
	 numImage= [NSImage imageNamed:[NSString stringWithFormat:@"numero-%d", num]];
	else 
	 numImage = [NSImage imageNamed:[NSString stringWithFormat:@"numero-bkg.png", num]];
	[numImage compositeToPoint:pt operation: NSCompositeSourceOver];
}

//range 0-99
-(void) setNumericDisplayValue:(int)newValue{
 if(newValue <0)
  newValue=0;
 if(newValue >99)
  newValue=99;
  
 nValore=newValue;
 [[self controlView] setNeedsDisplay:YES];
}
-(void) setDisplayNumericVisible:(BOOL)bYesNo{
bHaveNumericDisplay=bYesNo;
}

//Imposto il titolo
-(void) setTitle:(NSString *) aString{
 [theTitleRow1 release];
 theTitleRow1=[aString retain];
 [[self controlView] setNeedsDisplay:YES];
}
 
//Imposto il sottotitolo 
-(void) setSubTitle:(NSString *) aString{
 [theTitleRow2 release];
 theTitleRow2=[aString retain];
 [[self controlView] setNeedsDisplay:YES];
} 

-(void) setOutLetIndicator:(FGHudLevelIndicator *)theIndicator{
 aIndicator=theIndicator;
 [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(indicatorValueChanged:) name:@"FGHudLevelIndicatorValueChanged" object:nil];
}

-(void) indicatorValueChanged:(NSNotification *) aNotification{
 //  FGHudLevelIndicator *theIndicator = [aNotification object];
   [self setNumericDisplayValue:[aIndicator intValue]]; 
   [[self controlView] setNeedsDisplay:YES];
}

@end
